
#include "framebuffer.h"
#include "math.h"
#include <iostream>
#include "scene.h"

#include <tiffio.h>

using namespace std;

FrameBuffer::FrameBuffer(int u0, int v0, int _w, int _h) : 
	Fl_Gl_Window(u0, v0, _w, _h, 0) {

	w = _w;
	h = _h;
	pix = new unsigned int[w*h];

}


void FrameBuffer::draw() {

	glDrawPixels(w, h, GL_RGBA, GL_UNSIGNED_BYTE, pix);

}

int FrameBuffer::handle(int event) {

	switch (event)
	{
	case FL_KEYBOARD: {
		KeyboardHandle();
		return 0;
	}
	case FL_MOVE: {
		int u = Fl::event_x();
		int v = Fl::event_y();
		cerr << u << " " << v << "      \r";
		return 0;
	}
	default:
		return 0;
	}
	return 0;
}

void FrameBuffer::KeyboardHandle() {
	int key = Fl::event_key();
	switch (key) {
	case FL_Left: {
		cerr << "INFO: pressed left" << endl;
		break;
	}
	default:
		cerr << "INFO: do not understand keypress" << endl;
		return;
	}

}

void FrameBuffer::SetBGR(unsigned int bgr) {

	for (int uv = 0; uv < w*h; uv++)
		pix[uv] = bgr;

}

void FrameBuffer::SetChecker(unsigned int col0, unsigned int col1, int csize) {

	for (int v = 0; v < h; v++) {
		for (int u = 0; u < w; u++) {
			int cu = u / csize;
			int cv = v / csize;
			if ((cu + cv) % 2)
				Set(u, v, col0);
			else
				Set(u, v, col1);
		}
	}

}

void FrameBuffer::DrawAARectangle(V3 tlc, V3 brc, unsigned int col) {

	int umin = (int)tlc[0];
	int umax = (int)brc[0];
	int vmin = (int)tlc[1];
	int vmax = (int)brc[1];
	for (int v = vmin; v <= vmax; v++) {
		for (int u = umin; u <= umax; u++) {
			Set(u, v, col);
		}
	}

}

void FrameBuffer::DrawDisk(V3 center, float r, unsigned int col) {

	int umin = (int)(center[0] - r);
	int umax = (int)(center[0] + r);
	int vmin = (int)(center[1] - r);
	int vmax = (int)(center[1] + r);
	center[2] = 0.0f;
	for (int v = vmin; v <= vmax; v++) {
		for (int u = umin; u <= umax; u++) {
			V3 pixCenter(.5f + (float)u, .5f + (float)v, 0.0f);
			V3 distVector = pixCenter - center;
			if (r*r < distVector * distVector)
				continue;
			Set(u, v, col);
		}
	}


}


void FrameBuffer::Set(int u, int v, int col) {

	pix[(h - 1 - v)*w + u] = col;

}
